/* Copyright 1997 Lucent Technologies Inc. - All Rights Reserved */

#include <stdio.h>
#include <sys/types.h>
#include <sys/uio.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <signal.h>

#include "client.h"
#include "crypto.h"
#include "util.h"

void
log(char *msg, ...) {
	va_list args;

	va_start(args, msg);
	vfprintf(stderr, msg, args);
	fprintf(stderr, "\n");
	va_end(args);
}

/* milliseconds since the dawn of time. */
u_long
rtime(void) {
	struct timeval tp;
	static u_long start = 0;

	if (gettimeofday(&tp, 0) < 0)
		perror("gettimeofday");
	if (start == 0)
		start = tp.tv_sec;
	return (tp.tv_sec - start)*1000 + tp.tv_usec/1000;
}

static void *
timeout(void) {
	return;
}

char host_addr[20];	/* IP address we connected to */

int
tcpconnect(char *host, int port) {
	struct hostent *hp;
	struct sockaddr_in sin;
	unsigned long addr;
	int on = 1;
	int s, i;

	signal(SIGALRM, (void *)timeout);
	alarm(2*60);

	if (debug > 2)
		fprintf(stderr, "Connecting to server %s:%d\n",
			host, port);

	if ((addr = inet_addr(host)) != (unsigned long) -1) {
		s = socket(AF_INET, SOCK_STREAM, 0);
		if(s < 0)
			return -1;
		memset(&sin, 0, sizeof(sin));
		sin.sin_family = AF_INET;
		sin.sin_port = htons(port);
		sin.sin_addr.s_addr = addr;
		if (connect(s, (struct sockaddr *)&sin, sizeof (sin)) >= 0) {
			strncpy(host_addr, inet_ntoa(sin.sin_addr),
				sizeof(host_addr));
			alarm(0);
			if (setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof(on)) < 0)
				return -1;
			return s;
		}
		close(s);
		return -1;
	}

	if((hp = gethostbyname(host)) == (struct hostent *)0){
		/*fprintf(stderr, "unknown host: %s\n", host);*/
		return -1;
	}

	for(i=0; hp->h_addr_list[i]; i++){
		memset(&sin, 0, sizeof(sin));
		memcpy(&sin.sin_addr, hp->h_addr_list[i], hp->h_length);
		sin.sin_family = hp->h_addrtype;
		sin.sin_port = htons(port);

		s = socket(AF_INET, SOCK_STREAM, 0);
		if(s < 0)
			return -1;

		if(connect(s, (struct sockaddr *)&sin, sizeof (sin)) >= 0) {
			strncpy(host_addr, inet_ntoa(sin.sin_addr),
				sizeof(host_addr));
			alarm(0);
			if (setsockopt(s, SOL_SOCKET, SO_KEEPALIVE, &on, sizeof(on)) < 0)
				return -1;
		       return s;
		}
		close(s);
	}
	return -1;
}

/*
 * cservice file format:
 *
 *  service  host	params
 *
 * strtok is set up for the parameters
 */
char *
get_service_host(char *srvfile, char *service) {
	char *cp = findservice(srvfile, service);
	if (cp) {
		fprintf(stderr, "Can't find %s in %s: %s\n",
			service, srvfile, cp);
		exit(1);
	}
	cp = strtok(0, " \t\n");
	if (cp == 0) {
		fprintf(stderr, "Host missing for service %s in file %s\n",
			service, srvfile);
		exit(1);
	}
	return cp;
}
